@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# $Id: autogen.sh,v 1.3 2004/01/21 22:51:49 jcdutton Exp $
+# $Id: autogen.sh 844 2005-05-03 20:11:34Z mroi $
 #
 # This file is shamelessly copied from xine-lib.
 # With regards to Stephen Torri <storri@users.sourceforge.net> who
@@ -90,13 +90,14 @@ run_autoconf () {
 #--------------------
 # LIBTOOL
 #-------------------
-detect_libtool() {
-  set -- `type libtool 2>/dev/null`
+try_libtool_executable() {
+  libtool=$1
+  set -- `type $libtool 2>/dev/null`
   RETVAL=$?
   NUM_RESULT=$#
   RESULT_FILE=$3
   if [ $RETVAL -eq 0 -a $NUM_RESULT -eq 3 -a -f "$RESULT_FILE" ]; then
-    LT="`libtool --version | awk '{ print $4 }' | sed -e 's/[a-zA-Z\ \.\(\)\-\;]//g'`"
+    LT="`$libtool --version | awk '{ print $4 }' | sed -e 's/[a-zA-Z\ \.\(\)\-\;]//g'`"
     LIBTOOL_MIN="`echo $LIBTOOL_MIN | sed -e 's/[a-zA-Z\ \.\(\)\-]//g'`"
     if test $LT -lt 100 ; then
       LT=`expr $LT \* 10`
@@ -104,12 +105,21 @@ detect_libtool() {
     if [ `expr $LT` -ge $LIBTOOL_MIN ]; then
       libtool_ok=yes
     fi
-  else
-    echo
-    echo "**Error**: You must have \`libtool' >= $LIBTOOL_MIN installed to" 
-    echo "           compile $PROG. Download the appropriate package"
-    echo "           for your distribution or source from ftp.gnu.org."
-    exit 1
+  fi
+}
+
+detect_libtool() {
+  # try glibtool first, then libtool
+  try_libtool_executable 'glibtool'
+  if [ "x$libtool_ok" != "xyes" ]; then
+    try_libtool_executable 'libtool'
+    if [ "x$libtool_ok" != "xyes" ]; then
+      echo
+      echo "**Error**: You must have \`libtool' >= $LIBTOOL_MIN installed to" 
+      echo "           compile $PROG. Download the appropriate package"
+      echo "           for your distribution or source from ftp.gnu.org."
+      exit 1
+    fi
   fi
 }
 
@@ -122,7 +132,7 @@ run_libtoolize() {
   fi
 
   echo $_echo_n " + Running libtoolize: $_echo_c";
-    libtoolize --force --copy >/dev/null 2>&1;
+    "${libtool}ize" --force --copy >/dev/null 2>&1;
   echo "done."
 }
 
@@ -270,6 +280,6 @@ case "$1" in
     run_autoheader
     run_automake
     run_autoconf
-    run_configure $@
+    run_configure "$@"
     ;;
 esac
