@@ -1,6 +1,7 @@
+dvdreadlib="-ldvdread"
+dvdreadmsg="[--minilibs]"
 if test "$dvdread" = "external"; then
-    dvdreadlib="-ldvdread"
-    dvdreadmsg="[--minilibs --minicflags]"
+    dvdreadmsg="[--minicflags]"
     dvdreadcflags="-I$dvdreaddir"
     extracflags="-DDVDNAV_USES_EXTERNAL_DVDREAD"
 fi
@@ -50,11 +51,7 @@ while test $# -gt 0; do
       echo_libs=yes
       ;;
     --minilibs)
-      if test "$dvdread" = "external"; then
           echo_minilibs=yes
-      else
-          usage 1 1>&2
-      fi
       ;;
     *)
       usage 1 1>&2
@@ -68,7 +65,7 @@ if test "$echo_prefix" = "yes"; then
 fi
 
 if test "$echo_cflags" = "yes"; then
-      echo -I$prefix/include -I$prefix/include/dvdnav $dvdreadcflags $extracflags $threadcflags
+      echo -I$prefix/include $dvdreadcflags $extracflags $threadcflags
 fi
 
 if test "$echo_minicflags" = "yes"; then
@@ -76,9 +73,9 @@ if test "$echo_minicflags" = "yes"; then
 fi
 
 if test "$echo_libs" = "yes"; then
-      echo -L$prefix/lib -ldvdnav $dvdreadlib $threadlib
+      echo -L$libdir -ldvdnav $dvdreadlib $threadlib
 fi      
 
 if test "$echo_minilibs" = "yes"; then
-      echo -L$prefix/lib -ldvdnavmini $threadlib
+      echo -L$libdir -ldvdnavmini $threadlib
 fi
