@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: dvdnav.c,v 1.63 2004/01/31 17:12:58 jcdutton Exp $
+ * $Id: dvdnav.c 940 2007-06-11 21:14:12Z nicodvb $
  *
  */
 
@@ -29,14 +29,25 @@
 #define LOG_DEBUG
 */
 
+#include <inttypes.h>
+#include <stdlib.h>
+#include <stdio.h>
+#include <unistd.h>
+#include <limits.h>
+#include <string.h>
+#include <sys/time.h>
+#include "dvd_types.h"
+#include "dvd_reader.h"
+#include "nav_types.h"
+#include "ifo_types.h" /* For vm_cmd_t */
+#include "remap.h"
+#include "vm/decoder.h"
+#include "vm/vm.h"
+#include "dvdnav.h"
+#include "dvdnav_events.h"
 #include "dvdnav_internal.h"
 #include "read_cache.h"
 #include "nav_read.h"
-
-#include <stdlib.h>
-#include <stdio.h>
-#include <sys/time.h>
-
 #include "remap.h"
 
 static dvdnav_status_t dvdnav_clear(dvdnav_t * this) {
@@ -56,6 +67,7 @@ static dvdnav_status_t dvdnav_clear(dvdnav_t * this) {
   this->sync_wait_skip = 0;
   this->spu_clut_changed = 0;
   this->started = 0;
+  this->cur_cell_time = 0;
 
   dvdnav_read_cache_clear(this->cache);
   
@@ -121,11 +133,6 @@ dvdnav_status_t dvdnav_close(dvdnav_t *this) {
   fprintf(MSG_OUT, "libdvdnav: close:called\n");
 #endif
 
-  if(!this) {
-    printerr("Passed a NULL pointer.");
-    return DVDNAV_STATUS_ERR;
-  }
-
   if (this->file) {
     DVDCloseFile(this->file);
 #ifdef LOG_DEBUG
@@ -158,11 +165,6 @@ dvdnav_status_t dvdnav_reset(dvdnav_t *this) {
   fprintf(MSG_OUT, "libdvdnav: reset:called\n");
 #endif
 
-  if(!this) {
-    printerr("Passed a NULL pointer.");
-    return DVDNAV_STATUS_ERR;
-  }
-
   pthread_mutex_lock(&this->vm_lock); 
 
 #ifdef LOG_DEBUG
@@ -183,12 +185,6 @@ dvdnav_status_t dvdnav_reset(dvdnav_t *this) {
 }
 
 dvdnav_status_t dvdnav_path(dvdnav_t *this, const char** path) {
-
-  if(!this || !path) {
-    printerr("Passed a NULL pointer.");
-    return DVDNAV_STATUS_ERR;
-  }
-
   (*path) = this->path;
 
   return DVDNAV_STATUS_OK;
@@ -203,7 +199,7 @@ const char* dvdnav_err_to_string(dvdnav_t *this) {
 }
 
 /* converts a dvd_time_t to PTS ticks */
-static int64_t dvdnav_convert_time(dvd_time_t *time) {
+int64_t dvdnav_convert_time(dvd_time_t *time) {
   int64_t result;
   int64_t frames;
   
@@ -388,22 +384,37 @@ dvdnav_status_t dvdnav_get_next_block(dvdnav_t *this, uint8_t *buf,
   }
   return status;
 }
+
+int64_t dvdnav_get_current_time(dvdnav_t *this) {
+  int i;
+  int64_t tm=0;
+  dvd_state_t *state = &this->vm->state;
+
+  for(i=0; i<state->cellN-1; i++) {
+    if(!
+        (state->pgc->cell_playback[i].block_type == BLOCK_TYPE_ANGLE_BLOCK &&
+         state->pgc->cell_playback[i].block_mode != BLOCK_MODE_FIRST_CELL)
+    )
+      tm += dvdnav_convert_time(&state->pgc->cell_playback[i].playback_time);
+  }
+  tm += this->cur_cell_time;
+
+  return tm;
+}
  
 dvdnav_status_t dvdnav_get_next_cache_block(dvdnav_t *this, uint8_t **buf,
 					    int32_t *event, int32_t *len) {
   dvd_state_t *state;
   int32_t result;
 
-  if(!this || !event || !len || !buf || !*buf) {
-    printerr("Passed a NULL pointer.");
-    return DVDNAV_STATUS_ERR;
-  }
-
   pthread_mutex_lock(&this->vm_lock);
   
   if(!this->started) {
     /* Start the VM */
-    vm_start(this->vm);
+    if (!vm_start(this->vm)) {
+      printerr("Encrypted or faulty DVD");
+      return DVDNAV_STATUS_ERR;
+    }
     this->started = 1;
   }
 
@@ -580,6 +591,7 @@ dvdnav_status_t dvdnav_get_next_cache_block(dvdnav_t *this, uint8_t **buf,
     int32_t first_cell_nr, last_cell_nr, i;
     dvd_state_t *state = &this->vm->state;
     
+    this->cur_cell_time = 0;
     (*event) = DVDNAV_CELL_CHANGE;
 #ifdef LOG_DEBUG
     fprintf(MSG_OUT, "libdvdnav: CELL_CHANGE\n");
@@ -665,16 +677,10 @@ dvdnav_status_t dvdnav_get_next_cache_block(dvdnav_t *this, uint8_t **buf,
     fprintf(MSG_OUT, "libdvdnav: SPU_STREAM_CHANGE stream_id_wide=%d\n",stream_change->physical_wide);
     fprintf(MSG_OUT, "libdvdnav: SPU_STREAM_CHANGE stream_id_letterbox=%d\n",stream_change->physical_letterbox);
     fprintf(MSG_OUT, "libdvdnav: SPU_STREAM_CHANGE stream_id_pan_scan=%d\n",stream_change->physical_pan_scan);
+    fprintf(MSG_OUT, "libdvdnav: SPU_STREAM_CHANGE returning DVDNAV_STATUS_OK\n");
 #endif
-    if (stream_change->physical_wide != -1 &&
-        stream_change->physical_letterbox != -1 &&
-        stream_change->physical_pan_scan != -1) {
-#ifdef LOG_DEBUG
-      fprintf(MSG_OUT, "libdvdnav: SPU_STREAM_CHANGE returning DVDNAV_STATUS_OK\n");
-#endif
-      pthread_mutex_unlock(&this->vm_lock); 
-      return DVDNAV_STATUS_OK;
-    }
+    pthread_mutex_unlock(&this->vm_lock); 
+    return DVDNAV_STATUS_OK;
   }
 
   /* has the audio channel changed? */  
@@ -786,6 +792,7 @@ dvdnav_status_t dvdnav_get_next_cache_block(dvdnav_t *this, uint8_t **buf,
     fprintf(MSG_OUT, "libdvdnav: NAV_PACKET\n");
 #endif
     (*len) = 2048; 
+    this->cur_cell_time = dvdnav_convert_time(&this->dsi.dsi_gi.c_eltm);
     pthread_mutex_unlock(&this->vm_lock); 
     return DVDNAV_STATUS_OK;
   }
@@ -812,23 +819,13 @@ dvdnav_status_t dvdnav_get_next_cache_block(dvdnav_t *this, uint8_t **buf,
 }
 
 dvdnav_status_t dvdnav_get_title_string(dvdnav_t *this, const char **title_str) {
-  
-  if(!this || !title_str) {
-    printerr("Passed a NULL pointer.");
-    return DVDNAV_STATUS_ERR;
-  }
-
   (*title_str) = this->vm->dvd_name;
   return DVDNAV_STATUS_OK;
 }
 
 uint8_t dvdnav_get_video_aspect(dvdnav_t *this) {
   uint8_t         retval;
-  
-  if(!this) {
-    printerr("Passed a NULL pointer.");
-    return -1;
-  }
+
   if(!this->started) {
     printerr("Virtual DVD machine not started.");
     return -1;
@@ -844,10 +841,6 @@ uint8_t dvdnav_get_video_aspect(dvdnav_t *this) {
 uint8_t dvdnav_get_video_scale_permission(dvdnav_t *this) {
   uint8_t         retval;
   
-  if(!this) {
-    printerr("Passed a NULL pointer.");
-    return -1;
-  }
   if(!this->started) {
     printerr("Virtual DVD machine not started.");
     return -1;
@@ -863,10 +856,6 @@ uint8_t dvdnav_get_video_scale_permission(dvdnav_t *this) {
 uint16_t dvdnav_audio_stream_to_lang(dvdnav_t *this, uint8_t stream) {
   audio_attr_t  attr;
   
-  if(!this) {
-    printerr("Passed a NULL pointer.");
-    return -1;
-  }
   if(!this->started) {
     printerr("Virtual DVD machine not started.");
     return -1;
@@ -882,13 +871,62 @@ uint16_t dvdnav_audio_stream_to_lang(dvdnav_t *this, uint8_t stream) {
   return attr.lang_code;
 }
 
+uint16_t dvdnav_audio_stream_format(dvdnav_t *this, uint8_t stream) {
+  audio_attr_t  attr;
+  uint16_t format;
+  
+  if(!this->started) {
+    printerr("Virtual DVD machine not started.");
+    return -1; /* 0xffff */
+  }
+  
+  pthread_mutex_lock(&this->vm_lock); 
+  attr = vm_get_audio_attr(this->vm, stream);
+  pthread_mutex_unlock(&this->vm_lock); 
+  
+  switch(attr.audio_format) {
+  case 0:
+    format = DVDNAV_FORMAT_AC3;
+    break;
+  case 2: /* MPEG-1 or MPEG-2 without extension bitstream. */
+  case 3: /* MPEG-2 with extension bitstream. */
+    format = DVDNAV_FORMAT_MPEGAUDIO;
+    break;
+  case 4:
+    format = DVDNAV_FORMAT_LPCM;
+    break;
+  case 6:
+    format = DVDNAV_FORMAT_DTS;
+    break;
+  case 7:
+    format = DVDNAV_FORMAT_SDDS;
+    break;
+  default: 
+    format = 0xffff;
+    break;
+  }
+  
+  return format;
+}
+
+uint16_t dvdnav_audio_stream_channels(dvdnav_t *this, uint8_t stream) {
+  audio_attr_t  attr;
+
+  if(!this->started) {
+    printerr("Virtual DVD machine not started.");
+    return -1; /* 0xffff */
+  }
+
+  pthread_mutex_lock(&this->vm_lock);
+  attr = vm_get_audio_attr(this->vm, stream);
+  pthread_mutex_unlock(&this->vm_lock);
+
+  return attr.channels + 1;
+}
+
 uint16_t dvdnav_spu_stream_to_lang(dvdnav_t *this, uint8_t stream) {
   subp_attr_t  attr;
   
-  if(!this) {
-    printerr("Passed a NULL pointer.");
-    return -1;
-  }
   if(!this->started) {
     printerr("Virtual DVD machine not started.");
     return -1;
@@ -907,10 +945,6 @@ uint16_t dvdnav_spu_stream_to_lang(dvdnav_t *this, uint8_t stream) {
 int8_t dvdnav_get_audio_logical_stream(dvdnav_t *this, uint8_t audio_num) {
   int8_t       retval;
   
-  if(!this) {
-    printerr("Passed a NULL pointer.");
-    return -1;
-  }
   if(!this->started) {
     printerr("Virtual DVD machine not started.");
     return -1;
@@ -928,13 +962,26 @@ int8_t dvdnav_get_audio_logical_stream(dvdnav_t *this, uint8_t audio_num) {
   return retval;
 }
 
+dvdnav_status_t dvdnav_get_audio_attr(dvdnav_t *this, uint8_t audio_num, audio_attr_t *audio_attr) {
+  if(!this->started) {
+    printerr("Virtual DVD machine not started.");
+    return -1;
+  }
+  pthread_mutex_lock(&this->vm_lock);
+  if (!this->vm->state.pgc) {
+    printerr("No current PGC.");
+    pthread_mutex_unlock(&this->vm_lock); 
+    return -1;
+  }
+  *audio_attr=vm_get_audio_attr(this->vm, audio_num);
+  pthread_mutex_unlock(&this->vm_lock);
+
+  return DVDNAV_STATUS_OK;
+}
+
 int8_t dvdnav_get_spu_logical_stream(dvdnav_t *this, uint8_t subp_num) {
   int8_t       retval;
 
-  if(!this) {
-    printerr("Passed a NULL pointer.");
-    return -1;
-  }
   if(!this->started) {
     printerr("Virtual DVD machine not started.");
     return -1;
@@ -952,13 +999,25 @@ int8_t dvdnav_get_spu_logical_stream(dvdnav_t *this, uint8_t subp_num) {
   return retval;
 }
 
+dvdnav_status_t dvdnav_get_spu_attr(dvdnav_t *this, uint8_t audio_num, subp_attr_t *subp_attr) {
+  if(!this->started) {
+    printerr("Virtual DVD machine not started.");
+    return -1;
+  }
+  pthread_mutex_lock(&this->vm_lock);
+  if (!this->vm->state.pgc) {
+    printerr("No current PGC.");
+    pthread_mutex_unlock(&this->vm_lock); 
+    return -1;
+  }
+  *subp_attr=vm_get_subp_attr(this->vm, audio_num);
+  pthread_mutex_unlock(&this->vm_lock);
+  return DVDNAV_STATUS_OK;
+}
+
 int8_t dvdnav_get_active_audio_stream(dvdnav_t *this) {
   int8_t        retval;
 
-  if(!this) {
-    printerr("Passed a NULL pointer.");
-    return -1;
-  }
   if(!this->started) {
     printerr("Virtual DVD machine not started.");
     return -1;
@@ -979,10 +1038,6 @@ int8_t dvdnav_get_active_audio_stream(dvdnav_t *this) {
 int8_t dvdnav_get_active_spu_stream(dvdnav_t *this) {
   int8_t        retval;
 
-  if(!this) {
-    printerr("Passed a NULL pointer.");
-    return -1;
-  }
   if(!this->started) {
     printerr("Virtual DVD machine not started.");
     return -1;
@@ -1003,10 +1058,6 @@ int8_t dvdnav_get_active_spu_stream(dvdnav_t *this) {
 static int8_t dvdnav_is_domain(dvdnav_t *this, domain_t domain) {
   int8_t        retval;
   
-  if(!this) {
-    printerr("Passed a NULL pointer.");
-    return -1;
-  }
   if(!this->started) {
     printerr("Virtual DVD machine not started.");
     return -1;
@@ -1040,11 +1091,6 @@ int8_t dvdnav_is_domain_vts(dvdnav_t *this) {
 dvdnav_status_t dvdnav_angle_change(dvdnav_t *this, int32_t angle) {
   int32_t num, current;
   
-  if(!this) {
-    printerr("Passed a NULL pointer.");
-    return DVDNAV_STATUS_ERR;
-  }
-
   pthread_mutex_lock(&this->vm_lock);
   vm_get_angle_info(this->vm, &current, &num);
   /* Set angle SPRM if valid */
@@ -1062,11 +1108,6 @@ dvdnav_status_t dvdnav_angle_change(dvdnav_t *this, int32_t angle) {
 
 dvdnav_status_t dvdnav_get_angle_info(dvdnav_t *this, int32_t *current_angle,
 				      int32_t *number_of_angles) {
-  if(!this || !current_angle || !number_of_angles) {
-    printerr("Passed a NULL pointer.");
-    return DVDNAV_STATUS_ERR;
-  }
-
   pthread_mutex_lock(&this->vm_lock);
   vm_get_angle_info(this->vm, current_angle, number_of_angles);
   pthread_mutex_unlock(&this->vm_lock);
@@ -1089,3 +1130,30 @@ uint32_t dvdnav_get_next_still_flag(dvdnav_t *this) {
   return this->position_next.still;
 }
 
+user_ops_t dvdnav_get_restrictions(dvdnav_t* this) {
+  /* 
+   * user_ops_t is a structure of 32 bits.  We want to compute 
+   * the union of two of those bitfields so to make this quicker 
+   * than performing 32 ORs, we will access them as 32bits words.
+   */
+  union {
+    user_ops_t ops_struct;
+    uint32_t   ops_int;
+  } ops;
+  
+  ops.ops_int = 0;
+  
+  if(!this->started) {
+    printerr("Virtual DVD machine not started.");
+    return ops.ops_struct;
+  }
+  
+  pthread_mutex_lock(&this->vm_lock); 
+  ops.ops_int |= *(uint32_t*)&this->pci.pci_gi.vobu_uop_ctl;
+  
+  if(this->vm && this->vm->state.pgc)
+    ops.ops_int |= *(uint32_t*)&this->vm->state.pgc->prohibited_ops;
+  pthread_mutex_unlock(&this->vm_lock); 
+    
+  return ops.ops_struct;
+}
