@@ -227,14 +227,17 @@ static dvd_reader_t *DVDOpenImageFile( const char *location, int have_css )
     dev = dvdinput_open( location );
     if( !dev ) {
 	fprintf( stderr, "libdvdread: Can't open %s for reading\n", location );
-	return 0;
+	return NULL;
     }
 
     dvd = (dvd_reader_t *) malloc( sizeof( dvd_reader_t ) );
-    if( !dvd ) return 0;
+    if( !dvd ) {
+        dvdinput_close(dev);
+        return NULL;
+    }
     dvd->isImageFile = 1;
     dvd->dev = dev;
-    dvd->path_root = 0;
+    dvd->path_root = NULL;
     
     dvd->udfcache_level = DEFAULT_UDF_CACHE_LEVEL;
     dvd->udfcache = NULL;
@@ -256,10 +259,14 @@ static dvd_reader_t *DVDOpenPath( const char *path_root )
     dvd_reader_t *dvd;
 
     dvd = (dvd_reader_t *) malloc( sizeof( dvd_reader_t ) );
-    if( !dvd ) return 0;
+    if( !dvd ) return NULL;
     dvd->isImageFile = 0;
     dvd->dev = 0;
     dvd->path_root = strdup( path_root );
+    if(!dvd->path_root) {
+      free(dvd);
+      return 0;
+    }
 
     dvd->udfcache_level = DEFAULT_UDF_CACHE_LEVEL;
     dvd->udfcache = NULL;
@@ -321,7 +328,7 @@ dvd_reader_t *DVDOpen( const char *ppath )
     int ret;
     int have_css;
 	dvd_reader_t *ret_val = NULL;
-    char *dev_name = 0;
+    char *dev_name = NULL;
 	char *path;
 
 #ifdef _MSC_VER
@@ -364,7 +371,7 @@ dvd_reader_t *DVDOpen( const char *ppath )
 	fprintf( stderr, "libdvdread: Can't stat %s\n", path );
 	perror("");
 	free(path);
-	return 0;
+	return NULL;
     }
 
     /* First check if this is a block/char device or a file*/
@@ -398,7 +405,7 @@ dvd_reader_t *DVDOpen( const char *ppath )
 	/* XXX: We should scream real loud here. */
 	if( !(path_copy = strdup( path ) ) ) {
 		free(path);	
-		return 0;
+		return NULL;
 	}
 
 #ifndef WIN32 /* don't have fchdir, and getcwd( NULL, ... ) is strange */
@@ -411,13 +418,16 @@ dvd_reader_t *DVDOpen( const char *ppath )
 	    
 	    if( cdir >= 0 ) {
 		chdir( path_copy );
-		new_path = getcwd( NULL, PATH_MAX );
+		new_path = malloc(PATH_MAX+1);
+		if(!new_path) {
+		  free(path);
+		  return NULL;
+		}
+		getcwd(new_path, PATH_MAX );
 		fchdir( cdir );
 		close( cdir );
-		if( new_path ) {
 		    free( path_copy );
 		    path_copy = new_path;
-		}
 	    }
 	}
 #endif	
@@ -437,6 +447,11 @@ dvd_reader_t *DVDOpen( const char *ppath )
 	      path_copy[ strlen( path_copy ) - 9 ] = '\0';
 	    }
 	}
+	
+	if(path_copy[0] == '\0') {
+	    path_copy[0] = '/';
+	    path_copy[1] = '\0';
+	}
 
 #if defined(SYS_BSD)
 	if( ( fe = getfsfile( path_copy ) ) ) {
@@ -527,7 +542,7 @@ dvd_reader_t *DVDOpen( const char *ppath )
     /* If it's none of the above, screw it. */
     fprintf( stderr, "libdvdread: Could not open %s\n", path );
 	free( path );
-    return 0;
+    return NULL;
 }
 
 void DVDClose( dvd_reader_t *dvd )
@@ -551,13 +566,13 @@ static dvd_file_t *DVDOpenFileUDF( dvd_reader_t *dvd, char *filename )
     start = UDFFindFile( dvd, filename, &len );
     if( !start ) {
       fprintf( stderr, "libdvdnav:DVDOpenFileUDF:UDFFindFile %s failed\n", filename );
-      return 0;
+      return NULL;
     }
 
     dvd_file = (dvd_file_t *) malloc( sizeof( dvd_file_t ) );
     if( !dvd_file ) {
       fprintf( stderr, "libdvdnav:DVDOpenFileUDF:malloc failed\n" );
-      return 0;
+      return NULL;
     }
     dvd_file->dvd = dvd;
     dvd_file->lb_start = start;
@@ -641,19 +656,20 @@ static dvd_file_t *DVDOpenFilePath( dvd_reader_t *dvd, char *filename )
     /* Get the full path of the file. */
     if( !findDVDFile( dvd, filename, full_path ) ) {
       fprintf( stderr, "libdvdnav:DVDOpenFilePath:findDVDFile %s failed\n", filename );
-      return 0;
+      return NULL;
     }
 
     dev = dvdinput_open( full_path );
     if( !dev ) {
       fprintf( stderr, "libdvdnav:DVDOpenFilePath:dvdinput_open %s failed\n", full_path );
-      return 0;
+      return NULL;
     }
 
     dvd_file = (dvd_file_t *) malloc( sizeof( dvd_file_t ) );
     if( !dvd_file ) {
       fprintf( stderr, "libdvdnav:DVDOpenFilePath:dvd_file malloc failed\n" );
-      return 0;
+      dvdinput_close(dev);
+      return NULL;
     }
     dvd_file->dvd = dvd;
     dvd_file->lb_start = 0;
@@ -665,7 +681,7 @@ static dvd_file_t *DVDOpenFilePath( dvd_reader_t *dvd, char *filename )
     if( stat( full_path, &fileinfo ) < 0 ) {
         fprintf( stderr, "libdvdread: Can't stat() %s.\n", filename );
         free( dvd_file );
-        return 0;
+        return NULL;
     }
     dvd_file->title_sizes[ 0 ] = fileinfo.st_size / DVD_VIDEO_LB_LEN;
     dvd_file->title_devs[ 0 ] = dev;
@@ -686,10 +702,10 @@ static dvd_file_t *DVDOpenVOBUDF( dvd_reader_t *dvd, int title, int menu )
         sprintf( filename, "/VIDEO_TS/VTS_%02d_%d.VOB", title, menu ? 0 : 1 );
     }
     start = UDFFindFile( dvd, filename, &len );
-    if( start == 0 ) return 0;
+    if( start == 0 ) return NULL;
 
     dvd_file = (dvd_file_t *) malloc( sizeof( dvd_file_t ) );
-    if( !dvd_file ) return 0;
+    if( !dvd_file ) return NULL;
     dvd_file->dvd = dvd;
     /*Hack*/ dvd_file->css_title = title << 1 | menu;
     dvd_file->lb_start = start;
@@ -732,7 +748,7 @@ static dvd_file_t *DVDOpenVOBPath( dvd_reader_t *dvd, int title, int menu )
     int i;
 
     dvd_file = (dvd_file_t *) malloc( sizeof( dvd_file_t ) );
-    if( !dvd_file ) return 0;
+    if( !dvd_file ) return NULL;
     dvd_file->dvd = dvd;
     /*Hack*/ dvd_file->css_title = title << 1 | menu;
     dvd_file->lb_start = 0;
@@ -751,19 +767,20 @@ static dvd_file_t *DVDOpenVOBPath( dvd_reader_t *dvd, int title, int menu )
         }
         if( !findDVDFile( dvd, filename, full_path ) ) {
             free( dvd_file );
-            return 0;
+            return NULL;
         }
 
         dev = dvdinput_open( full_path );
         if( dev == NULL ) {
             free( dvd_file );
-            return 0;
+            return NULL;
         }
 
         if( stat( full_path, &fileinfo ) < 0 ) {
             fprintf( stderr, "libdvdread: Can't stat() %s.\n", filename );
+            dvdinput_close(dev);
             free( dvd_file );
-            return 0;
+            return NULL;
         }
         dvd_file->title_sizes[ 0 ] = fileinfo.st_size / DVD_VIDEO_LB_LEN;
         dvd_file->title_devs[ 0 ] = dev;
@@ -790,7 +807,7 @@ static dvd_file_t *DVDOpenVOBPath( dvd_reader_t *dvd, int title, int menu )
         }
         if( !dvd_file->title_devs[ 0 ] ) {
             free( dvd_file );
-            return 0;
+            return NULL;
         }
     }
 
@@ -1079,6 +1096,7 @@ int DVDDiscID( dvd_reader_t *dvd, unsigned char *discid )
 {
     struct md5_ctx ctx;
     int title;
+    int nr_of_files = 0;
 
     /* Check arguments. */
     if( dvd == NULL || discid == NULL )
@@ -1113,9 +1131,12 @@ int DVDDiscID( dvd_reader_t *dvd, unsigned char *discid )
 	    
 	    DVDCloseFile( dvd_file );
 	    free( buffer_base );
+	    nr_of_files++;
 	}
     }
     md5_finish_ctx( &ctx, discid );
+    if(!nr_of_files)
+      return -1;
     
     return 0;
 }
