@@ -13,7 +13,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: ifo_print.c 952 2007-08-02 21:58:59Z nicodvb $
+ * $Id: ifo_print.c 980 2007-12-30 13:59:19Z nicodvb $
  *
  */
 
@@ -210,24 +210,51 @@ static void ifo_print_audio_attributes(int level, audio_attr_t *attr) {
   switch(attr->audio_format) {
   case 0:
     printf("ac3 ");
+    if(attr->quantization != 3)
+      printf("(please send a bug report) ac3 quant/drc not 3 (%d)", attr->quantization);
     break;
   case 1:
     printf("(please send a bug report) ");
     break;
   case 2:
     printf("mpeg1 ");
-    break;
   case 3:
     printf("mpeg2ext ");
+    switch(attr->quantization) {
+      case 0:
+        printf("no drc ");
+        break;
+      case 1:
+        printf("drc ");
+        break;
+      default:
+        printf("(please send a bug report) mpeg reserved quant/drc  (%d)", attr->quantization);  
+    }
     break;
   case 4:
     printf("lpcm ");
+    switch(attr->quantization) {
+      case 0:
+        printf("16bit ");
+        break;
+      case 1:
+        printf("20bit ");
+        break;
+      case 2:
+        printf("24bit ");
+        break;
+      case 3:
+        printf("(please send a bug report) lpcm reserved quant/drc  (%d)", attr->quantization);
+      break;
+    }
     break;
   case 5:
     printf("(please send a bug report) ");
     break;
   case 6:
     printf("dts ");
+    if(attr->quantization != 3)
+      printf("(please send a bug report) dts quant/drc not 3 (%d)", attr->quantization);
     break;
   default:
     printf("(please send a bug report) ");
@@ -1064,7 +1091,10 @@ void ifo_print(dvd_reader_t *dvd, int title) {
     ifo_print_VMGI_MAT(ifohandle->vmgi_mat);
 
     printf("\nFirst Play PGC\n--------------\n");
-    ifo_print_PGC(ifohandle->first_play_pgc);
+    if(ifohandle->first_play_pgc)
+      ifo_print_PGC(ifohandle->first_play_pgc);
+    else
+      printf("No First Play PGC present\n");
 
     printf("\nTitle Track search pointer table\n");
     printf(  "------------------------------------------------\n");
