@@ -19,7 +19,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: decoder.c 900 2007-04-24 23:06:22Z nicodvb $
+ * $Id: decoder.c 993 2008-01-05 09:22:45Z nicodvb $
  *
  */
 
@@ -417,7 +417,7 @@ static int32_t eval_system_set(command_t* command, int32_t cond, link_t *return_
    For the swap case the contents of reg is stored in reg2.
 */
 static void eval_set_op(command_t* command, int32_t op, int32_t reg, int32_t reg2, int32_t data) {
-  const int32_t shortmax = 0xffff;
+  static const int32_t shortmax = 0xffff;
   int32_t     tmp; 
   switch(op) {
     case 1:
